@@ -23,7 +23,7 @@ Animation and VFX Studios.
 See docs for more information.
 """
 
-__version__ = '0.2.5.5'
+__version__ = '0.2.6'
 
 
 # before anything about stalker create the defaults
@@ -57,6 +57,7 @@ from stalker.models.template import FilenameTemplate
 from stalker.models.ticket import Ticket, TicketLog
 from stalker.models.type import Type, EntityType
 from stalker.models.version import Version
+from stalker.models.wiki import Page
 from stalker.models.auth import group_finder
 
 import logging
