@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2013 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
@@ -79,15 +79,16 @@ class Config(object):
 
         actions=['Create', 'Read', 'Update', 'Delete', 'List'],  #CRUDL
 
-        status_bg_color=0xffffff,
-        status_fg_color=0x000000,
-
         # Tickets
         ticket_label="Ticket",
 
         # define the available actions per Status
-        ticket_status_order=[
-            'new', 'accepted', 'assigned', 'reopened', 'closed'
+        ticket_status_names=[
+            'New', 'Accepted', 'Assigned', 'Reopened', 'Closed'
+        ],
+
+        ticket_status_codes=[
+            'NEW', 'ACP', 'ASG', 'ROP', 'CLS'
         ],
 
         ticket_resolutions=[
@@ -96,62 +97,62 @@ class Config(object):
 
         ticket_workflow={
             'resolve': {
-                'new': {
-                    'new_status': 'closed',
+                'New': {
+                    'new_status': 'Closed',
                     'action': 'set_resolution'
                 },
-                'accepted': {
-                    'new_status': 'closed',
+                'Accepted': {
+                    'new_status': 'Closed',
                     'action': 'set_resolution'
                 },
-                'assigned': {
-                    'new_status': 'closed',
+                'Assigned': {
+                    'new_status': 'Closed',
                     'action': 'set_resolution'
                 },
-                'reopened': {
-                    'new_status': 'closed',
+                'Reopened': {
+                    'new_status': 'Closed',
                     'action': 'set_resolution'
                 },
             },
             'accept': {
-                'new': {
-                    'new_status': 'accepted',
+                'New': {
+                    'new_status': 'Accepted',
                     'action': 'set_owner'
                 },
-                'accepted': {
-                    'new_status': 'accepted',
+                'Accepted': {
+                    'new_status': 'Accepted',
                     'action': 'set_owner'
                 },
-                'assigned': {
-                    'new_status': 'accepted',
+                'Assigned': {
+                    'new_status': 'Accepted',
                     'action': 'set_owner'
                 },
-                'reopened': {
-                    'new_status': 'accepted',
+                'Reopened': {
+                    'new_status': 'Accepted',
                     'action': 'set_owner'
                 },
             },
             'reassign': {
-                'new': {
-                    'new_status': 'assigned',
+                'New': {
+                    'new_status': 'Assigned',
                     'action': 'set_owner'
                 },
-                'accepted': {
-                    'new_status': 'assigned',
+                'Accepted': {
+                    'new_status': 'Assigned',
                     'action': 'set_owner'
                 },
-                'assigned': {
-                    'new_status': 'assigned',
+                'Assigned': {
+                    'new_status': 'Assigned',
                     'action': 'set_owner'
                 },
-                'reopened': {
-                    'new_status': 'assigned',
+                'Reopened': {
+                    'new_status': 'Assigned',
                     'action': 'set_owner'
                 },
             },
             'reopen': {
-                'closed': {
-                    'new_status': 'reopened',
+                'Closed': {
+                    'new_status': 'Reopened',
                     'action': 'del_resolution'
                 }
             }
@@ -159,7 +160,6 @@ class Config(object):
 
         # Task Management
         timing_resolution=datetime.timedelta(hours=1),
-        task_duration=datetime.timedelta(hours=1),
 
         task_priority=500,
 
@@ -176,9 +176,9 @@ class Config(object):
         # this is strongly related with the working_hours settings,
         # this should match each other
         daily_working_hours=9,
-        weekly_working_hours=45,
         weekly_working_days=5,
-        yearly_working_days=260.714,  # 5 * 52.1428
+        weekly_working_hours=45,
+        yearly_working_days=261,  # math.ceil(5 * 52.1428)
 
         day_order=['mon', 'tue', 'wed', 'thu', 'fri', 'sat', 'sun'],
 
@@ -194,82 +194,153 @@ class Config(object):
             'y'  : {'name': 'days'   , 'multiplier': 365}
         },
 
-        task_schedule_models=['effort', 'length', 'duration'],
+        task_status_names=[
+            'Waiting For Dependency',
+            'Ready To Start',
+            'Work In Progress',
+            'Pending Review',
+            'Has Revision',
+            'Dependency Has Revision',
+            'On Hold',
+            'Stopped',
+            'Completed'
+        ],
+
+        task_status_codes=[
+            'WFD',
+            'RTS',
+            'WIP',
+            'PREV',
+            'HREV',
+            'DREV',
+            'OH',
+            'STOP',
+            'CMPL'
+        ],
+
+        review_status_names=[
+            'New',
+            'Requested Revision',
+            'Approved'
+        ],
+        review_status_codes=[
+            'NEW',
+            'RREV',
+            'APP'
+        ],
 
         task_schedule_constraints=['none', 'start', 'end', 'both'],
+        task_schedule_models=['effort', 'length', 'duration'],
+        task_dependency_gap_models=['length', 'duration'],
+        task_dependency_targets=['onend', 'onstart'],
+        allocation_strategy=['minallocated', 'maxloaded', 'minloaded', 'order',
+                             'random'],
+        persistent_allocation=True,
 
-        tjp_working_hours_template="""{% macro wh(wh, day) -%}
-        {%- if wh[day]|length %}    workinghours {{day}} {% for part in wh[day] -%}
-                {%- if loop.index != 1%}, {% endif -%}
-                {{"%02d"|format(part[0]//60)}}:{{"%02d"|format(part[0]%60)}} - {{"%02d"|format(part[1]//60)}}:{{"%02d"|format(part[1]%60)}}
-                {%- endfor -%}
-        {%- else %}    workinghours {{day}} off
-        {%- endif -%}
-        {%- endmacro -%}
-        {{wh(workinghours, 'mon')}}
-        {{wh(workinghours, 'tue')}}
-        {{wh(workinghours, 'wed')}}
-        {{wh(workinghours, 'thu')}}
-        {{wh(workinghours, 'fri')}}
-        {{wh(workinghours, 'sat')}}
-        {{wh(workinghours, 'sun')}}""",
+        tjp_working_hours_template="""
+{%- macro wh(wh, day) -%}
+{%- if wh[day]|length %}    workinghours {{day}} {% for part in wh[day] -%}
+    {%- if loop.index != 1%}, {% endif -%}
+        {{"%02d"|format(part[0]//60)}}:{{"%02d"|format(part[0]%60)}} - {{"%02d"|format(part[1]//60)}}:{{"%02d"|format(part[1]%60)}}
+    {%- endfor -%}
+{%- else %}    workinghours {{day}} off
+{%- endif -%}
+{%- endmacro -%}
+{{wh(workinghours, 'mon')}}
+{{wh(workinghours, 'tue')}}
+{{wh(workinghours, 'wed')}}
+{{wh(workinghours, 'thu')}}
+{{wh(workinghours, 'fri')}}
+{{wh(workinghours, 'sat')}}
+{{wh(workinghours, 'sun')}}""",
 
-        tjp_studio_template="""project {{ studio.tjp_id }} "{{ studio.name }}" {{ studio.start.date() }} - {{ studio.end.date() }} {
-            timingresolution {{ '%i'|format((studio.timing_resolution.days * 86400 + studio.timing_resolution.seconds)//60|int) }}min
-            now {{ studio.now.strftime('%Y-%m-%d-%H:%M') }}
-            dailyworkinghours {{ studio.daily_working_hours }}
-            weekstartsmonday
-        {{ studio.working_hours.to_tjp }}
-            timeformat "%Y-%m-%d"
-            scenario plan "Plan"
-            trackingscenario plan
-        }
-        """,
+        tjp_studio_template="""
+project {{ studio.tjp_id }} "{{ studio.name }}" {{ studio.start.date() }} - {{ studio.end.date() }} {
+    timingresolution {{ '%i'|format((studio.timing_resolution.days * 86400 + studio.timing_resolution.seconds)//60|int) }}min
+    now {{ studio.now.strftime('%Y-%m-%d-%H:%M') }}
+    dailyworkinghours {{ studio.daily_working_hours }}
+    weekstartsmonday
+{{ studio.working_hours.to_tjp }}
+    timeformat "%Y-%m-%d"
+    scenario plan "Plan"
+    trackingscenario plan
+}
+""",
 
-        tjp_project_template="""task {{project.tjp_id}} "{{project.name}}" {
-            {% for task in project.root_tasks %}
-                {{task.to_tjp}}
-            {% endfor %}
-        }
-        """,
+        tjp_project_template="""
+task {{project.tjp_id}} "{{project.name}}" {
+    {% for task in project.root_tasks %}
+        {{task.to_tjp}}
+    {%- endfor %}
+}
+""",
 
-        tjp_task_template="""task {{task.tjp_id}} "{{task.name}}" {
-        {% if task.priority != 500 -%}priority {{task.priority}}{%- endif %}
-        {%- if task.depends %}
-            depends {% for depends in task.depends %}
-            {%- if loop.index != 1 %}, {% endif %}{{depends.tjp_abs_id}}
-        {%- endfor -%}
-        {%- endif -%}
-        {%- if task.is_container -%}
-            {%- for child_task in task.children %}
-                {{ child_task.to_tjp }}
-            {%- endfor %}
-        {%- else %}
-            {% if task.resources|length -%}
-            {% if task.schedule_constraint %}
-                {%- if task.schedule_constraint == 1 or task.schedule_constraint == 3 -%}
-                    start {{ task.start.strftime('%Y-%m-%d-%H:%M') }}
-                {%- endif %}
-                {%- if task.schedule_constraint == 2 or task.schedule_constraint == 3 %}
-                    end {{ task.end.strftime('%Y-%m-%d-%H:%M') }}
-                {%- endif -%}
-            {% endif %}
-            {{task.schedule_model}} {{task.schedule_timing}}{{task.schedule_unit}}
-            allocate {% for resource in task.resources -%}
-                {%-if loop.index != 1 %}, {% endif %}{{resource.tjp_id}}{% endfor %}
-            {%- endif -%}
-            {% for time_log in task.time_logs %}
-            booking {{time_log.resource.tjp_id}} {{time_log.start.strftime('%Y-%m-%d-%H:%M:%S')}} +{{'%i'|format(time_log.duration.days*24 + time_log.duration.seconds/3600)}}h { overtime 2 }
-            {%- endfor -%}
-        {% endif %}
-        }
-        """,
+        tjp_task_template="""
+task {{task.tjp_id}} "{{task.name}}" {
 
-        tjp_department_template='''resource {{department.tjp_id}} "{{department.name}}" {
-        {%- for resource in department.users %}
-            {{resource.to_tjp}}
+    {% if task.priority != 500 -%}
+        priority {{task.priority}}
+    {%- endif %}
+
+    {% if task.task_depends_to %}
+        depends {# #}
+        {%- for depends in task.task_depends_to %}
+            {%- if loop.index != 1 %}, {% endif %}{{depends.to_tjp}}
         {%- endfor %}
-        }''',
+    {%- endif -%}
+
+    {%- if task.is_container -%}
+        {% for child_task in task.children -%}
+            {{ child_task.to_tjp }}
+        {%- endfor %}
+    {%- else %}
+        {% if task.resources|length %}
+
+            {% if task.schedule_constraint %}
+                {% if task.schedule_constraint == 1 or task.schedule_constraint == 3 %}
+                    start {{ task.start.strftime('%Y-%m-%d-%H:%M') }}
+                {% endif %}
+                {% if task.schedule_constraint == 2 or task.schedule_constraint == 3 -%}
+                    end {{ task.end.strftime('%Y-%m-%d-%H:%M') }}
+                {% endif %}
+            {% endif %}
+
+            {{task.schedule_model}} {{task.schedule_timing}}{{task.schedule_unit}}
+            allocate {# #}
+            {%- for resource in task.resources -%}
+                {%- if loop.index != 1 %}, {% endif %}{{ resource.tjp_id }} {# #}
+                {%- if task.alternative_resources %}{
+                    alternative
+                    {% for alt_res in task.alternative_resources -%}
+                        {%-if loop.index != 1 %}, {% endif %}
+                        {{- alt_res.tjp_id}}
+                    {%- endfor -%}
+                    {# #} select {{task.allocation_strategy}}
+                    {% if task.persistent_allocation -%}
+                        persistent
+{# #}
+                    {%- endif %}
+                }
+                {%- endif %}
+            {%- endfor %}
+        {%- endif %}
+        {% for time_log in task.time_logs %}
+            booking {{time_log.resource.tjp_id}} {{time_log.start.strftime('%Y-%m-%d-%H:%M:%S')}} +{{'%i'|format(time_log.duration.days*24 + time_log.duration.seconds/3600)}}h { overtime 2 }
+        {%- endfor %}
+    {% endif %}
+
+}
+""",
+
+        tjp_task_dependency_template="""{{depends_to.tjp_abs_id}} { {{- dependency_target}}{%if gap_timing %} gap{{gap_model}} {{gap_timing}}{{gap_unit -}}{%endif -%}}""",
+
+        tjp_department_template="""
+resource {{department.tjp_id}} "{{department.name}}" {
+{% for resource in department.users %}
+    {{resource.to_tjp}}
+{% endfor -%}
+}
+""",
 
         tjp_vacation_template='''vacation {{ vacation.start.strftime('%Y-%m-%d-%H:%M:%S') }} - {{ vacation.end.strftime('%Y-%m-%d-%H:%M:%S') }}''',
 
@@ -301,7 +372,7 @@ class Config(object):
         taskreport breakdown "{{csv_file_full_path}}"{
             formats csv
             timeformat "%Y-%m-%d-%H:%M"
-            columns id, start, end
+            columns id, start, end, resources
         }
         """,
 
@@ -413,8 +484,10 @@ class Config(object):
                 logger.warning("The $STALKER_PATH: %s doesn't exists! "
                                "skipping user config" % resolved_path)
             except SyntaxError, err:
-                raise RuntimeError("There is a syntax error in your "
-                                   "configuration file: %s" % str(err))
+                raise RuntimeError(
+                    "There is a syntax error in your configuration file: %s" %
+                    str(err)
+                )
             finally:
                 # append the data to the current settings
                 logger.debug("updating system config")
