@@ -224,7 +224,7 @@ class TaskJugglerSchedulerTester(unittest2.TestCase):
         import jinja2
 
         expected_tjp_template = jinja2.Template(
-            """# Generated By Stalker v0.2.5.5
+            """# Generated By Stalker v0.2.6
         
 project Studio_564 "Test Studio" 2013-04-16 - 2013-06-30 {
     timingresolution 30min
